package algorithms;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import structures.Node;
import structures.ScenariosTree;
import structures.Transition;

public class DimacsCnfBuilder {

    public static String getCnf(ScenariosTree tree, int k) {
        Map<String, Integer> vars = new HashMap<String, Integer>();
        for (Node node : tree.getNodes()) {
            for (int color = 0; color < k; color++) {
                vars.put("x_" + node.getNumber() + "_" + color, vars.size() + 1);
            }
        }
        
        for (Node node : tree.getNodes()) {
            for (Transition t : node.getTransitions()) {
                String key = "y_" + t.getEvent() + "_" + t.getExpr().toString() + "_0_0";
                if (!vars.containsKey(key)) {
                    for (int nodeColor = 0; nodeColor < k; nodeColor++) {
                        for (int childColor = 0; childColor < k; childColor++) {
                            String s = "y_" + t.getEvent() + "_" + t.getExpr() + "_" + nodeColor + "_" + childColor;
                            vars.put(s, vars.size() + 1);
                        }
                    }
                }
            }
        }
        
        List<String> clauses = new ArrayList<String>();
        String initClause = vars.get("x_0_0") + "";
        clauses.add(initClause);
        
        for (Node node : tree.getNodes()) {
            String clause = "";
            for (int color = 0; color < k; color++) {
                clause += vars.get("x_" + node.getNumber() + "_" + color) + " "; 
            }
            clauses.add(clause);
        }
        
        for (Node node : tree.getNodes()) {
            for (int c1 = 0; c1 < k; c1++) {
                for (int c2 = 0; c2 < c1; c2++) {
                    int v1 = vars.get("x_" + node.getNumber() + "_" + c1);
                    int v2 = vars.get("x_" + node.getNumber() + "_" + c2);
                    clauses.add(-v1 + " " + -v2);
                }
            }
        }
        
        Map<Node, Set<Node>> adjacent = AdjacentCalculator.getAdjacent(tree);
        for (Node node : tree.getNodes()) {
            for (Node other : adjacent.get(node)) {
                if (other.getNumber() < node.getNumber()) {
                    for (int color = 0; color < k; color++) {
                        int v1 = vars.get("x_" + node.getNumber() + "_" + color);
                        int v2 = vars.get("x_" + other.getNumber() + "_" + color);
                        clauses.add("-" + v1 + " -" + v2);                      
                    }
                }
            }
        }
        
        Set<String> was = new HashSet<String>();
        for (Node node : tree.getNodes()) {
            for (Transition t : node.getTransitions()) {
                String key = t.getEvent() + "_" + t.getExpr();
                if (!was.contains(key)) {
                    was.add(key);
                    for (int parentColor = 0; parentColor < k; parentColor++) {
                        for (int c1 = 0; c1 < k; c1++) {
                            for (int c2 = 0; c2 < c1; c2++) {
                                int v1 = vars.get("y_" + key + "_" + parentColor + "_" + c1);
                                int v2 = vars.get("y_" + key + "_" + parentColor + "_" + c2);
                                clauses.add(-v1 + " " + -v2);
                            }
                        }
                    }
                }
            }
        }
        
        for (Node node : tree.getNodes()) {
            for (Transition t : node.getTransitions()) {
                for (int nodeColor = 0; nodeColor < k; nodeColor++) {
                    for (int childColor = 0; childColor < k; childColor++) {
                        int nodeVar = vars.get("x_" + node.getNumber() + "_" + nodeColor);
                        int childVar = vars.get("x_" + t.getDst().getNumber() + "_" + childColor);
                        int realtionVar = vars.get("y_" + t.getEvent() + "_" + t.getExpr() + "_" + nodeColor + "_" + childColor);
                        
                        clauses.add(realtionVar + " " + -nodeVar + " " + -childVar);
                        clauses.add(-realtionVar + " " + -nodeVar + " " + childVar);
                    }
                }
            }
        }
        
        StringBuilder sb = new StringBuilder();
        sb.append("c CNF for scenarios tree with " + tree.nodesCount() + " nodes, colored in " + k + " colors\n");
        sb.append("p cnf " + vars.size() + " " + clauses.size() + "\n");
        for (String s : clauses) {
            sb.append(s + " 0\n");
        }
        
        return sb.toString();
    }
}
