package algorithms;

import java.util.Random;

import structures.Automaton;
import structures.Node;
import structures.Transition;

public class ScenariosGenerator {
	static public String generateScenarios(Automaton automaton, int scenariosCount, int minLength, int maxLength,
			int sumLength, Random random) {

		int[] length = getRandomLength(scenariosCount, minLength, maxLength, sumLength, random);

		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < scenariosCount; i++) {
			if (i > 0) {
				sb.append("\n");
			}
			sb.append(generateScenario(automaton, length[i], random));
		}
		return sb.toString();
		
	}

	static private String generateScenario(Automaton automaton, int length, Random random) {
		String events = "", actions = "";

		Node curNode = automaton.getStartState();
		for (int i = 0; i < length; i++) {
			if (i > 0) {
				events += "; ";
				actions += "; ";
			}

			if (curNode.getTransitions().isEmpty()) {
				throw new RuntimeException("There is no outcoming transitions from node number " + curNode.getNumber());
			}
			int transitionNum = random.nextInt(curNode.getTransitions().size());
			Transition transition = curNode.getTransitions().toArray(new Transition[0])[transitionNum];

			events += transition.getEvent() + "[" + transition.getExpr() + "]";
			actions += transition.getActions();

			curNode = transition.getDst();
		}

		return events + "\n" + actions + "\n";
	}

	static private int[] getRandomLength(int scenariosCount, int minLength, int maxLength, int sumLength, Random random) {

		assert 0 < minLength && minLength <= maxLength;
		assert minLength * scenariosCount <= sumLength;
		assert sumLength <= maxLength * scenariosCount;

		int[] length = new int[scenariosCount];
		for (int i = 0; i < scenariosCount; i++) {
			length[i] = sumLength / scenariosCount;
		}
		for (int i = 0; i < sumLength % scenariosCount; i++) {
			length[i]++;
		}

		for (int i = 0; i < scenariosCount; i++) {
			int s1 = random.nextInt(scenariosCount);
			int s2 = random.nextInt(scenariosCount);

			int maxDiff = Math.min(length[s1] - minLength, maxLength - length[s2]);
			if (maxDiff > 0) {
				int diff = 1 + random.nextInt(maxDiff);
				length[s1] -= diff;
				length[s2] += diff;
			}
		}

		return length;
	}

}
