package scenario;

import java.io.File;
import java.io.FileNotFoundException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import bool.MyBooleanExpression;
import actions.StringActions;

public class StringScenario {
	
	public static List<StringScenario> loadScenarios(String filepath) throws ParseException, FileNotFoundException {
		List<StringScenario> ans = new ArrayList<StringScenario>();
		
		Scanner in = new Scanner(new File(filepath));

		String inp = "";
		while (in.hasNextLine()) {
			String s = in.nextLine().trim();
			if (inp.equals("") && s.equals("")) {
				continue;
			}

			if (inp.equals("")) {
				inp = s;
			} else {
				ans.add(new StringScenario(inp, s));
				inp = "";
			}
		}

		in.close();
		return ans;
	}
	
    private String[] events;
    
    private MyBooleanExpression[] expressions;
    
    private StringActions[] actions;
    
    public StringScenario(String input, String output) throws ParseException {
        String[] events = input.split(";");
        String[] actions = (output + " ").split(";");
        if (actions.length != events.length) {
            throw new ParseException("events length " + events.length + " != actions length " + actions.length, 0);
        }
         
        int n = actions.length;
        
        this.events = new String[n];
        this.expressions = new MyBooleanExpression[n];
        this.actions = new StringActions[n];
        
        for (int i = 0; i < n; i++) {
            MyBooleanExpression expr;
            
            if (events[i].contains("[")) {
                String[] p = events[i].split("\\[");
                this.events[i] = p[0].trim();
                expr = MyBooleanExpression.get(p[1].replace(']', ' '));
            } else {
                this.events[i] = events[i].trim();
                expr = MyBooleanExpression.get("1");            
            }
            
            this.expressions[i] = expr;
            
            this.actions[i] = new StringActions(actions[i]);
        }

    }
    
    public int size() {
        return events.length;
    }
    
    public String getEvent(int pos) {
        return events[pos];
    }
    
    public MyBooleanExpression getExpr(int pos) {
        return expressions[pos];
    }
    
    public StringActions getActions(int pos) {
        return actions[pos];
    }
    
    public String toString() {
        String inp = "";
        String out = "";
        for (int i = 0; i < size(); i++) {
            if (i > 0) {
                inp += "; ";
                out += "; ";
            }
            inp += events[i] + "[" + expressions[i].toString() + "]";
            out += actions[i].toString();
        }
        return inp + "\n" + out;
    }
}
