import sys
from pylab import *
import json

def main(results_fp):
    results = json.load(open(results_fp))
    
    points = {}
    for r in results:
        size, sum_length = r['size'], r['sum_length']
        time, isomorph = r['time'], 1 if r['is_isomorph'] else 0
        if size not in points:
            points[size] = {}
        if sum_length not in points[size]:
            points[size][sum_length] = ([], [])
        time_ls, iso_ls = points[size][sum_length]
        points[size][sum_length] = (time_ls + [time], iso_ls + [isomorph])
    
    subplot = figure().add_subplot(1, 1, 1)

    for size in sorted(points.keys()):
        lens, times = [], []
        for sum_len in sorted(points[size].keys()):
            time_ls, iso_ls = points[size][sum_len]
            iso_avg = 1. * sum(iso_ls) / len(iso_ls)
            time_avg = sum(time_ls) / len(time_ls)
            print size, sum_len, iso_avg, time_avg
            
            lens.append(sum_len)
            times.append(time_avg)

        subplot.plot(lens, times, 'x-', color='black')
    
    sum_lengths = set()
    for s in points:
        sum_lengths.update(points[s].keys())
    xlabels = sorted(list(sum_lengths))
    subplot.set_xticks(xlabels)

    xlabel('Summary scenarious size')
    ylabel('CryptoMiniSat execution time (s)')
    
    image_fp = '.'.join(results_fp.split('.')[:-1]) + '.png'
    savefig(image_fp)
    show()

if __name__ == '__main__':
    if len(sys.argv) != 2:
        print 'Usage: %s <fp>' % sys.argv[0]
    else:
        main(sys.argv[1])
